//==============================================================================
// PROJECT      = L_Line - L2 demo
// MODULE       = LCD.c
// SHORT DESC.  = LCD driver routines
// DEVICE       = 78K0/LG2
// VERSION      = 1.0
// DATE         = 01.09.2005
// LAST CHANGE  =
// =============================================================================
// By:          NEC Electronics (Europe) GmbH
//              Arcadiastrasse 10
//              D-40472 Duesseldorf
// =============================================================================

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include "io78f0397.h"
#include "defines.h"
#include <intrinsics.h>

//-----------------------------------------------------------------------------
// Function prototyps
//-----------------------------------------------------------------------------
extern void Wait(unsigned char Number);
__callt void  LCD_cmd(char addr, char data);

//-----------------------------------------------------------------------------
// Extern variables/constants
//-----------------------------------------------------------------------------
extern const char *s_clear;
extern __saddr volatile unsigned char sw3_in;

//-----------------------------------------------------------------------------
// Global constants: minimum ASCII table for 14 segment LCD panel
//-----------------------------------------------------------------------------
const short characters[43]   = {0xe007, // '0'
                                0x0206, // '1'
                                0xc423, // '2'
                                0x8427, // '3'
                                0x2426, // '4'
                                0xa425, // '5'
                                0xe425, // '6'
                                0x0007, // '7'
                                0xe427, // '8'
                                0xa427, // '9'
                                0x05a0, // '+'
                                0x0420, // '-'
                                0x2423, // ''
                                0x8000, // '_'
                                0x0240, // '/'
                                0xc424, // 'o'	
                                0x0000, // ' '  => space
                                0x6427, // 'A'
                                0xe424, // 'B'
                                0xe001, // 'C'
                                0xc426, // 'D'
                                0xe021, // 'E'
                                0x6021, // 'F'
                                0xe405, // 'G'
                                0x6426, // 'H'
                                0x0180, // 'I'
                                0xc007, // 'J'
                                0x6a20, // 'K'
                                0xe000, // 'L'
                                0x6216, // 'M'
                                0x6816, // 'N'
                                0xe007, // 'O'
                                0x6423, // 'P'
                                0xe807, // 'Q'
                                0x6c23, // 'R'
                                0xa425, // 'S'
                                0x0181, // 'T'
                                0xe006, // 'U'
                                0x0816, // 'V'
                                0x6946, // 'W'
                                0x0a50, // 'X'
                                0x0290, // 'Y'
                                0x8241  // 'Z'
                               };

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
__saddr char trans_count=0;
__saddr volatile char flag2=0;
__saddr char i2c_status;
__saddr char transmit_buffer[5];
__saddr char message_byte_count;

///-----------------------------------------------------------------------------
// Module:   i2c_init
// Description: Initialisation of IIC bus for LCD controller communication
//-----------------------------------------------------------------------------
void  i2c_init (void)
{
 // Init IICCL0 register:
 // ---------------------
 SMC0  = 0;      // Operation mode switching.
                 // 0 = operation in standard mode.
                 // 1 = op. in high speed mode
 DFC0  = 1;      // Control of digital filter operation.
                 // 0 = digital filter ON, 1 = digital filter OFF
 // Init IICC0 register:
 // --------------------
 IICE0 = 1;      // I2C operation enable. enable = 1 disable = 0
 PM6 = 0xfc;
 LREL0 = 1;      // Set SCL0 and SDA0 in high impedace state and clear all
                 // staus flags in IICS0 register
 SPT0  = 1;      // Stop condition trigger.
                 // 0 = does not generate Stop condition   1 = generates Stop condtion
 SPIE0 = 0;      // Enable/disable generation of inter. when stop is
                 // detected. Disable = 0, enable = 1
 WTIM0 = 1;      // Control of wait and inter. request generation
                 // 0 = inter. at the eighth clocks fall. edge
                 // 1 = inter. at the ninth clocks fall. edge
 ACKE0 = 0;      // Auto Acknowledge trigger bit.
                 // 0 = No auto ACK.
                 // 1 = Enable Auto ACK.
 WREL0 = 1;      // Release in any case the Wait State.
 STT0  = 0;      // Start condition trigger.
                 // 0 = does not generate Start condition   1 = generates Start condtion
 while (!SPD0);  // Wait for Stop Detect bit to be set, before continue
 i2c_status = I2C_READY;
}

//-----------------------------------------------------------------------------
// Module:   LCD_init
// Description: Initialization of LCD controller
//-----------------------------------------------------------------------------
void LCD_init()
{
  i2c_init();           // setup IIC bus

  P13 =0x1;             // LCD reset status released
  CKS = 0x08;
  PM14 =0xfe;           // Clock output to LCD enabled
  CKS = 0x18;

  LCD_cmd (LCDMD, 0x02);     // Voltage boosting method
  LCD_cmd (LCDM , 0x00);
  LCD_cmd (LCDC , 0x0c);     // Set display frequency
  LCD_cmd (VLCG0 ,0x80);
  LCD_cmd (LCDM , 0x20);

  Wait(40);                   // voltage boost wait time = 2s

  LCD_cmd (LCDM , 0x60);
  LCD_cmd (LCDM , 0xE0);
}

//-----------------------------------------------------------------------------
// Module:   LCD_out
// Description: Transfer data to LCD controller
//-----------------------------------------------------------------------------
void  LCD_out(char lcdaddr)
{
    if (i2c_status == I2C_READY)    // start transmit
    {
          IICIF0 = 0;       // delete interrupt flag for IIC
          IICMK0 = 0 ;      // enable I2C interrupt

          trans_count=0;
          i2c_status   = I2C_MST_TRANSMISSION ;

          ACKE0 = 1;
          STT0 = 1 ;
          IIC0 = lcdaddr;   // start transfer of command or RAM data

    }
    while (!flag2);         // wait for flag2 => all data transmitted
    flag2=0;                // reset flag2

    IICMK0 = 1 ;            // disable I2C interrupt
}

//-----------------------------------------------------------------------------
// Module:   LCD_cmd
// Description: Sent command to the LCD controller
//-----------------------------------------------------------------------------
__callt void  LCD_cmd(char addr, char data)
{
    // load transmit buffer
    message_byte_count = 2;
    transmit_buffer [0] = addr;
    transmit_buffer [1] = data;
    LCD_out(LCD_CON);           // transfer command
}


//-----------------------------------------------------------------------------
// Module:   LCD_putc
// Description: Sent character to LCD controller
//-----------------------------------------------------------------------------
__callt void LCD_putc (char seg, char data)
{
    // convert special characters
    switch(data)
    {
      case 0x2f:  data = 0x0d;  // '_'
                  break;
      case 0xf0:  data = 0x10;  // ' ' => space
                  break;
      case 0x80:  data = 0x0c;  // ''
                  break;
      case 0xfb:  data = 0x0a;  // '+'
                  break;
      case 0xfd:  data = 0x0b;  // '-'
                  break;
      case 0xff:  data = 0x0e;  // '/'
                  break;
      case 0x3f:  data = 0x0f;  // 'o'
                  break;
      default:    break;
    }

    // load transmit buffer
    message_byte_count = 5;
    transmit_buffer [0] = seg<<2;
    transmit_buffer [1] = ((characters[data])& 0xf000)>>12;
    transmit_buffer [2] = ((characters[data])& 0x0f00)>>8;
    transmit_buffer [3] = ((characters[data])& 0x00f0)>>4;
    transmit_buffer [4] = ((characters[data])& 0x000f);
    LCD_out(LCD_RAM);     // transfer data
}

//-----------------------------------------------------------------------------
// Module:   LCD_string
// Description: Send character string to LCD module
//-----------------------------------------------------------------------------
__callt void LCD_string(char const *point, char dpos)
{
  while(dpos<=7)
  {
      if(point[0])
      {
          LCD_putc(dpos,*point-0x30);
          *point++;
      }
      else
      {
          LCD_putc(dpos,0xf0);
      }
      dpos++;
  }
}

//-----------------------------------------------------------------------------
// Module:   LCD_string_shift
// Description: Send character string to LCD module
//-----------------------------------------------------------------------------
void LCD_string_shift(char const *point)
{
  char dpos=7;
  while(dpos!=0xff)
  {
      if(sw3_in)
      {
        LCD_string(&s_clear[0],0);
        return;
      }
      LCD_string(point,dpos);
      dpos--;
      Wait(4);
  }
  *point++;
  dpos=0;
  while(point[0])
  {
      if(sw3_in)
      {
         LCD_string(&s_clear[0],0);
         return;
      }
      *point++;
      LCD_string(point,dpos);
      Wait(4);
  }
}

//-----------------------------------------------------------------------------
// Module:   isr_INTIIC0
// Description: IIC communication interrupt service routine
//-----------------------------------------------------------------------------
#pragma vector = INTIIC0_vect
__interrupt void isr_INTIIC0(void)
{
    if (i2c_status == I2C_MST_TRANSMISSION)
    {
      if (ACKD0)
      {
             if ((trans_count < message_byte_count))
                {
                  IIC0 = transmit_buffer[trans_count] ;
                  trans_count++;
                }
            else
                {
                  flag2 = 1;
                  SPT0 = 1 ;    // send stop condition
                  i2c_status = I2C_READY ;
                }
       }
       else                     // no acknowledge
       {
       flag2 = 1;
       SPT0 = 1 ;               // send stop condition
       i2c_status = I2C_READY ;
       }
     }
}

